/***************************************************************************
                          main.c  -  vaiod main file
                             -------------------
    begin                : Thu Aug  9 10:40:47 BST 2001
    copyright            : (C) 2001 by Lee Packham
    email                : lpackham@mswinxp.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Enable for non-daemon running
//#define DEBUG_MODE

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "vaioutils.h"
#include "execwrap.h"
#include "defines.h"

// Path to sh
#define _PATH_BSHELL "/bin/sh"

static pid_t mypid = (pid_t)-1;

pid_t sys_fork(void)
{
	pid_t forkret = fork();

	if (forkret == (pid_t)0)
		mypid = (pid_t)-1;

	return forkret;
}

int main(int argc, char *argv[])
{
	int curBatStatus = 0;
	int oldBatStatus = 0;

#ifdef CMD_BATTLOW
	int intBattLow = -1;
#endif

	int intWhTotal = 0;
	int intWhCurrent = 0;
	int intWhOldCurrent = 0;

	// Fork ourselves
#ifndef DEBUG_MODE
	if (sys_fork()) _exit(0);
	setsid();
	umask(0);

	// Close links with the console
	close(0); close(1); close(2);
#endif

	// Get current AC Status
	oldBatStatus = GetACStatus();

	// Get current Wh Status
	intWhOldCurrent = GetWhCurrent();

	// Set the brightness to 0 if we are on battery
	if (!oldBatStatus) SetBrightness(0);

	for (;;) {

		// If we have no power, we should turn off the backlight
		// If power is restored, turn the backlight on again!
		curBatStatus = GetACStatus();
		if (curBatStatus) {
			intBattLow = -1;

			if (!oldBatStatus)
				SetBrightness(255);
		}
		else {
			if (oldBatStatus)
				SetBrightness(0);
		}

		intWhTotal = GetWhTotal();
		intWhCurrent = GetWhCurrent();

#ifdef DEBUG_MODE
		printf("%d, %d\n", intWhCurrent, intWhTotal);
		printf("Old Current = %d\n", intWhOldCurrent);
		printf("Current Battery = %d\n", curBatStatus);
		printf("Old Bat = %d\n", oldBatStatus);
#endif

#ifdef CMD_BATTFULL
		// Has the batter gone from charging to full?
		if (intWhCurrent == intWhTotal && intWhCurrent > intWhOldCurrent && curBatStatus) {
			execProgram(CMD_BATTFULL);
		}
#endif

#ifdef CMD_BATTLOW
#ifdef BATTLOW_PERCENT
		// No AC and battery low?
		if (intWhCurrent / intWhTotal < BATTLOW_PERCENT && !curBatStatus && intBattLow) {
			execProgram(CMD_BATTLOW);

			// We have played the battery low
			intBattLow = 0;
		}
#endif
#endif

#ifdef CMD_BATTCRIT
#ifdef BATTCRIT_PERCENT
		// No AC and battery low?
		if (intWhCurrent / intWhTotal <= BATTCRIT_PERCENT && !curBatStatus) {
			execProgram(CMD_BATTCRIT);			
		}
#endif
#endif

		// Store the AC status thang
		oldBatStatus = curBatStatus;
		intWhOldCurrent = intWhCurrent;

		sleep(1);
	}

  return EXIT_SUCCESS;
}
