/***************************************************************************
                          vaioutils.c  -  description
                             -------------------
    begin                : Mon Aug 6 2001
    copyright            : (C) 2001 by Lee Packham
    email                : lpackham@mswinxp.net
 ***************************************************************************/

/*
	This file really remains with thanks to tridge@samba.org from whom
	I stole the register details and the like.
*/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/io.h>
#include <sys/mman.h>
#include <dirent.h>
#include <ctype.h>
#include <malloc.h>
#include <string.h>
#include <signal.h>
#include <getopt.h>
#include <linux/pci.h>
#include <sys/time.h>

#include "vaioutils.h"

#define DATA_REG 0x62
#define CST_REG  0x66

typedef unsigned short u16;
typedef unsigned char u8;

void ecr_set(u8 addr, u8 value)
{
        while (inb(CST_REG) & 3) usleep(1);
        outb(0x81, CST_REG);
        while (inb(CST_REG) & 2) usleep(1);
        outb(addr, DATA_REG);
        while (inb(CST_REG) & 2) usleep(1);
        outb(value, DATA_REG);
        while (inb(CST_REG) & 2) usleep(1);
}

u8 ecr_get(u8 addr)
{
        while (inb(CST_REG) & 3) usleep(1);
        outb(0x80, CST_REG);
        while (inb(CST_REG) & 2) usleep(1);
        outb(addr, DATA_REG);
        while (inb(CST_REG) & 2) usleep(1);
        return inb(DATA_REG);
}

u16 ecr_get16(u8 addr)
{
        return ecr_get(addr) | (ecr_get(addr+1)<<8);
}

int UIDValid()
{
	// Vaio stuff can only run as root, so we have to make sure we are root
	return (getuid() == 0) ? -1 : 0;
}

// Returns the brightness as a value between 0 - 255
int GetBrightness()
{
	u8 curBrightness;

	if (!UIDValid()) return -1;

	iopl(3);

	curBrightness = ecr_get(0x96);
	return curBrightness;	
}

void SetBrightness(int brightness)
{
	if (!UIDValid()) return;

	iopl(3);

	if (brightness < 0 || brightness > 255) {
		return;
	}
	else {
		ecr_set(0x96, brightness);
	}
}

int GetBat1Present()
{
	u16 batstatus;

	if (!UIDValid()) return -1;

	iopl(3);

	batstatus = ecr_get16(0x81);
	return (batstatus & 1) ? -1 : 0;
}

int GetBat2Present()
{
	u16 batstatus;

	if (!UIDValid()) return -1;

	iopl(3);

	batstatus = ecr_get16(0x81);
	return (batstatus & 2) ? -1 : 0;
}

int GetACStatus()
{
	u16 batstatus;

	if (!UIDValid()) return -1;

	iopl(3);

	batstatus = ecr_get16(0x81);
	return (batstatus & 0x4) ? -1 : 0;
}

int GetWhTotal()
{
	u16 whTotal;

	if (!UIDValid()) return -1;

	iopl(3);

	whTotal = ecr_get16(0xb0);
	return whTotal;
}

int GetWhCurrent()
{
	u16 whCurrent;
	
	if (!UIDValid()) return -1;

	iopl(3);

	whCurrent = ecr_get16(0xa2);
	return whCurrent;
}
